indexing
   title:       "Time"
   cluster:     "Time & Date - Timespan"
   project:     "SIG Moscow Cluster library"
   copyright:   "SIG Computer Inc, Moscow", 1995
   author:      "Alexei Simonov"
   approved_by: "Frieder Monninger"
   original:    "3,June,95"
   version:     "1.0"
   done_at:     "SIG Computer Inc, Moscow (info@eiffel.ru)"
   license:     "GNU Library GPL - see included readme file"


class TIME

inherit
   SIMPLE_TIMESTAMP
      redefine
         infix "<"
   end

   MATH_UTILS
         undefine
            is_equal
         end

creation
   clear,
   set_time,
   set_to_now,
   set_from_milliseconds


feature

   milliseconds  : INTEGER;

   set_time( hr, me : INTEGER; sd : REAL ) is
   require
      time_valid : tt.valid_time( hr, me, sd );
   do
      milliseconds :=
         real_to_int (( hr * tt.seconds_in_hour + me * tt.seconds_in_minute + sd ) *
         tt.milliseconds_in_second);
   end -- set_time


   set_to_now is
   local
      time : NOW
   do
      !!time;
      set_time( time.hour, time.minute, time.second );
   end -- set_to_now


   set_from_milliseconds ( m : INTEGER ) is
   require
      milliseconds_valid : m >= tt.zero and then
         m < tt.seconds_in_day * tt.milliseconds_in_second
   do
      milliseconds := m;
   end -- set_from_milliseconds



   clear is
   do
      milliseconds := tt.zero;
   end -- clear


   infix "<"( ts : like Current ) : BOOLEAN is

   do
      Result := milliseconds < ts.milliseconds
   end -- infix "<"


   infix "+",
   plus( d : SHORT_DURATION ) :  TIMESTAMP is
   local
      ms : INTEGER;
   do
      !!Result.set_from_compressed (tt.zero, milliseconds + d.milliseconds);
   end -- infix "+", plus


   infix "-",
   minus( d : SHORT_DURATION ) :  TIMESTAMP is
   do
      !!Result.set_from_compressed (tt.zero, milliseconds - d.milliseconds);
   end -- infix "-", minus


   difference( t : like Current ) : SHORT_DURATION is
   do
      !!Result.set_seconds ((milliseconds - t.milliseconds) / tt.milliseconds_in_second );
   end -- difference


   hour_part : INTEGER is
   do
      Result :=
         real_to_int (milliseconds / ( tt.milliseconds_in_second * tt.seconds_in_hour ));
   end -- hour_part


   minute_part : INTEGER is
   do
      Result :=
         real_to_int (( milliseconds / tt.milliseconds_in_second -
         hour_part * tt.seconds_in_hour ) / tt.seconds_in_minute );
   end -- minute_part


   second_part : REAL is
   local
      ms : REAL
   do
      ms := milliseconds;
      Result :=
         ms / tt.milliseconds_in_second -
         hour_part * tt.seconds_in_hour - minute_part * tt.seconds_in_minute;
   end -- second_part

invariant
  time_big_enough  :  milliseconds >=tt.zero
  time_small_enough:  milliseconds < tt.milliseconds_in_day

end -- class TIME

