indexing
   title:       "Long Duration - days or more"
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    22,Sep,95
   version:     "1.0"
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "long_drt.e";


class LONG_DURATION

   inherit
      SIMPLE_DURATION
         redefine
            infix "<"
         end;

   creation
      clear,
      set_days,
      set_date

   feature { ANY }

      absolute_days : INTEGER;

      set_date( yr, mh, dy : INTEGER ) is
         require
            date_valid : dy <= tt.days_in_month_d and then
                         dy >= - tt.days_in_month_d and then
                         mh <= tt.months_in_year and then
                         mh >= - tt.months_in_year
         do
            absolute_days := yr * tt.days_in_year_d +
                             mh * tt.days_in_month_d + dy;
         end -- set_date


      set_days( days : INTEGER ) is
         do
            absolute_days := days;
         end -- set_days


      clear is
         do
            absolute_days := tt.zero;
         end -- clear


      year_part : INTEGER is
         do
            Result := absolute_days // tt.days_in_year_d;
         end -- year_part

      month_part : INTEGER is
         do
            Result :=
               ( absolute_days - tt.days_in_year_d * year_part ) //
               tt.days_in_month_d;
         end -- month_part

      day_part : INTEGER is
         do
            Result :=
               absolute_days - tt.days_in_year_d * year_part -
               month_part * tt.days_in_month_d;
         end -- day_part

      as_days : REAL is
         do
            Result := absolute_days;
         end -- as_days

      as_months : REAL is
         local
            abs_day : REAL;
         do
            abs_day := absolute_days;
            Result  := abs_day / tt.days_in_month_d;
         end -- as_months

      as_years : REAL is
         local
            abs_day : REAL;
         do
            abs_day := absolute_days;
            Result := abs_day / tt.days_in_year_d;
         end -- as_years

      whole_days: INTEGER is
         do
            Result := real_to_int (as_days)
         end -- whole_days

      whole_months: INTEGER is
         do
            Result := real_to_int (as_months)
         end -- whole_months

      whole_years: INTEGER is
         do
            Result := real_to_int (as_years)
         end -- whole_years

      infix "<"( ts : like Current ) :  BOOLEAN is
         do
            Result := Current.absolute_days < ts.absolute_days;
         end -- infix "<"

      infix "+",
      plus ( d: like Current ) :  like Current is
         local
            sum : INTEGER;
         do
            sum := Current.absolute_days + d.absolute_days;
            !!Result.set_days( sum );
         end -- infix "+", plus

      infix "-",
      minus ( d: like Current ) :  like Current is
         local
            res : INTEGER;
         do
            res := Current.absolute_days - d.absolute_days;
            !!Result.set_days( res );
         end -- infix "-", minus


      multiply_by ( n: INTEGER ) : like Current is
         do
            !!Result.set_days( absolute_days * n );
         end -- multiply_by

      divide_by ( n: INTEGER ) : like Current is
         require else
            non_zero_divider : n /= tt.zero
         do
            !!Result.set_days( absolute_days // n );
         end -- divide_by

      ratio,
      divide_by_duration ( d: like Current ) : REAL is
         require else
            Non_Zero_param : d.absolute_days /= 0
         local
            abs_day : REAL
         do
            abs_day := absolute_days;
            Result := abs_day / d.absolute_days;
         end -- ratio, divide_by_duration

end -- class LONG_DURATION
